#========================================================================================
# package     CustomFormTest
# description Test depending on Store datbase
#========================================================================================
package CustomFormTest;
use base DE_EPAGES::Shop::API::Test;

use strict;

use Test::More tests => 10;
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );

sub init {
    my $self = shift;
    $self->SUPER::init;
    my $Shop = $self->shop;
    $Shop->featureMaxValue('CustomForm', 50); ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)

    return;
}

sub test {
    my $self = shift;
    my $Shop =$self->shop;
    my $Alias = 'MyCustomForm';
    my $hInfo = {Alias => $Alias, Parent => $Shop};
    my $Class = LoadClassByAlias('CustomForm');
    my $CustomForm = $Class->insertObject($hInfo);
    isa_ok($CustomForm, 'DE_EPAGES::CustomForm::API::Object::CustomForm');

    # Getter generates $CustomForm/Fields/EMail
    my $aList = $CustomForm->get('Fields');
    ok($aList->[0]->instanceOf('EMailFormField '), 'default EmailFormField is set');

    # Additional text field
    my $Folder = $CustomForm->folder('Fields');
    my $FieldClass = LoadClassByAlias('TextFormField');
    my $TextFormAlias = 'MyLovelyField';
    my $TextFormField = $Folder->insertChild($FieldClass, {
        'Alias'    => $TextFormAlias,
        'Position' => 11,
    });
    is($CustomForm->id, $TextFormField->get('Form')->id, 'CustomFormField->get(\'Form\')' );

    # We must change Alias, if we don't change the parent.
    my $DO_RECURSIVE = 1;
    my $Clone = $CustomForm->clone({Alias => 'ClonedCustomForm'}, $DO_RECURSIVE);
    isnt( $Clone->id, $CustomForm->id, 'clone is a new object' );
    is( $Clone->alias, 'ClonedCustomForm', 'clone has a new alias' );
    is( $Clone->parent->id, $CustomForm->parent->id, 'clone has the same parent' );

    # We can't use getter - it would generate child 'EMail' as default
    my $EMail = $Clone->path('Fields/EMail');
    is($EMail->alias, 'EMail', 'EMail field is cloned');

    # Compare the rest
    is( scalar @{$Clone->get('Fields')}, scalar @{$CustomForm->get('Fields')}, 'clone has the same number of fields' );
    $aList = $Clone->get('Fields');
    is($aList->[1]->alias, $TextFormAlias, "cloned Fields contains $TextFormAlias");

    $CustomForm->delete();
    ok(! $Shop->existsChild($Alias), 'delete CustomForm' );

}

__PACKAGE__->runScript;

1;
