#========================================================================================
# package      DE_EPAGES::EPagesJ::t::CommunicationLogger
# state        private
#========================================================================================
package DE_EPAGES::EPagesJ::t::CommunicationLogger;
use base Test::Class;

use strict;

use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::EPagesJ::API::Object::CommunicationLogger;
use HTTP::Request::Common qw ( POST );
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::MockObject;
use Test::More;

sub cleanup : Test(shutdown) {
    my $self = shift;
}

sub testLogCommunication : Test(no_plan) {
    my $self = shift;
    my $LogString = 'Dies ist ein commLog-Test';
    my $Logger = DE_EPAGES::EPagesJ::API::Object::CommunicationLogger->new(
            'isLogging' => 1,
    );
    ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    no warnings 'redefine';
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger = sub { return $self->_mockLogger($LogString); };
    $Logger->logCommunication($LogString);
}

sub testDummyLoggerByDisableLogging : Test(no_plan) {
    my $self = shift;
    my $LogString = 'Do not commlog me!';
    my $Logger = DE_EPAGES::EPagesJ::API::Object::CommunicationLogger->new(
            'isLogging' => 0,
    );
    ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    no warnings 'redefine';
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger = sub { fail('commlog should not be called.'); return $self->_mockLogger($LogString); };
    $Logger->logCommunication($LogString);
    pass('passed disable logging');
}

sub testLogHttpRequest : Test(no_plan) {
    my $self = shift;
    my $Logger = DE_EPAGES::EPagesJ::API::Object::CommunicationLogger->new(
            'isLogging' => 1,
    );
    ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    no warnings 'redefine';
    my $Request = POST('http://www.google.de/?q=abc', 'Content' => 'A=B');
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger = sub { return $self->_mockLogger($Request->as_string); };
    $Logger->logCommunication($Request);
}

sub testLogHttpResponse : Test(no_plan) {
    my $self = shift;
    my $LogString = 'xxx';
    my $Logger = DE_EPAGES::EPagesJ::API::Object::CommunicationLogger->new(
            'isLogging' => 1,
    );
    ## no critic (ProtectPrivateVars ProhibitNoWarnings)
    no warnings 'redefine';
    my $Response = HTTP::Response->new(HTTP_OK, status_message(HTTP_OK), undef, $LogString);
    local *DE_EPAGES::EPagesJ::API::Object::CommunicationLogger::logger = sub { return $self->_mockLogger($Response->as_string); };
    $Logger->logCommunication($Response);
}

sub assertLogContents {
    my ($Content, $ExpectedString, $level) = @_;
#    print "\n\nAA" . $Content . "ZZ\n\n";
#    print "\n\nAA" . $ExpectedString . "ZZ\n\n"; 
    is( $Content, $ExpectedString, $level . ' logging ' . length($Content) . ' chars.');
}


sub _mockLogger {
    my $self = shift;
    my ($assertedLogMessage) = @_;
    my $mock = Test::MockObject->new();
    my $assertedLogString = $assertedLogMessage;
    my %mockMethods = (
        'debug' => sub { shift; assertLogContents(shift, $assertedLogString, 'debug') },
        'is_debug' => sub {1},
    );
    while ( my ( $method, $handler ) = each %mockMethods ) {
        $mock->mock( $method, $handler );
    }

    return $mock;
}

####### Main ############
RunScript(
    Debug => 1,
    Sub   => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => sub {
                Test::Class->runtests;
            }
        );
    }
);
