#========================================================================================
# package      DE_EPAGES::Order::t::pdfInvoiceIntegrationInvoice
# state        private
#----------------------------------------------------------------------------------------
# description  Integration test for the PDF generation webservice.
#               Use environment variable "SEND_TESTMAIL_TO" to send an email
#               with the expected and actual PDF output via email for manual 
#               acceptence testing.
#========================================================================================
package DE_EPAGES::Order::t::pdfInvoiceIntegrationInvoice;
use base 'Test::Class';

use DE_EPAGES::Core::API::File qw ( GetFileContent GetFileInfo RunOnDirectory WriteFile );
use DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Mail::API::Mail;
use DE_EPAGES::Mail::API::SMTP;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath );
use File::Basename;
use File::Temp qw ( tempfile );
use Test::More;

use strict;

my $ACTUAL_PDF;
my $EXPECTED_PDF = 'pdfIntegrationExpected.pdf';
my $INPUT_HTML   = 'pdfIntegrationInvoice.html';
my $SEND_TESTMAIL_TO = $ENV{'SEND_TESTMAIL_TO'};

sub testGenerateInvoicePdf : Test(1) {
    my $self = shift;
    return 'DemoShop is missing!' unless (LoadRootObject->child('Shops')->existsChild('DemoShop'));
    my $Html      = ${ GetFileContent($INPUT_HTML, { utf8 => 1, }) };
    my $StoreRoot = LoadRootObject()->get('StoreRoot');
    $Html =~ s#/WebRoot/StoreTypes/6\.15\.4/Store#$StoreRoot#g;
    my $PdfContent = DE_EPAGES::EPagesJ::API::PDF::PdfClient->create(LoadObjectByPath('/Shops/DemoShop'))->transform($Html);
    (undef, $ACTUAL_PDF) = tempfile( 'SUFFIX' => '_pdfIntegrationActual.pdf', 'UNLINK' => 1);
    WriteFile( $ACTUAL_PDF, \$PdfContent, { 'binary' => 1 } );
    cmp_ok( abs( GetFileInfo($ACTUAL_PDF)->{Size} - GetFileInfo($EXPECTED_PDF)->{Size} ),
        '<', 1000, 'PDF file size difference smaller 1k' ) or sendComparisonViaMail();
}

sub sendComparisonViaMail {
    return unless ($SEND_TESTMAIL_TO);
    diag 'sending mail to ' . $SEND_TESTMAIL_TO;
    my $Connection = DE_EPAGES::Mail::API::SMTP->new( 'mail', 25 );
    $Connection->connection(
        sub {
            my $Mail = DE_EPAGES::Mail::API::Mail->new( {
                    'FROM'    => 'nobody@epages.com',
                    'TO'      => [$SEND_TESTMAIL_TO],
                    'SUBJECT' => 'PDF Webservice Integration Testmail'
                }
            );
            $Mail->addText(
                { 'content' => 'Generated by pdfIntegation.t. Please check for visual differences in PDF output.' } );

            $Mail->addAttachment( [ {
                        'filename' => $EXPECTED_PDF,
                    },
                    { 'filename' => $ACTUAL_PDF, }
                ]
            );
            $Mail->send($Connection);
        }
    );
}

####### Main ############
RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);
