#========================================================================================
# package      DE_EPAGES::Presentation::t::CurrencyHandler
# description  Testing CurrencyHandler TLE functions
#========================================================================================
package DE_EPAGES::Presentation::t::CurrencyHandler;

use strict;
use warnings;
use Test::More tests => 5;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Presentation::API::TLE::CurrencyHandler;

sub _TestWithViewAction {
    my $Handler = DE_EPAGES::Presentation::API::TLE::CurrencyHandler->new;
    my $ResultingURL = $Handler->ChangeCurrencyURL(MockedTLEProcessor->new({
        'INPUT.ViewAction' => 'ABC',
        'INPUT.ViewObjectID' => ''
    }), ['EUR']);

    is( $ResultingURL,
        '?ObjectPath=Products/%2208/15%22&ViewAction=ABC&Currency=EUR',
        '_TestWithViewAction'
    );
}

sub _TestWithoutViewAction {
    my $Handler = DE_EPAGES::Presentation::API::TLE::CurrencyHandler->new;
    my $ResultingURL = $Handler->ChangeCurrencyURL(MockedTLEProcessor->new({
        'INPUT.ViewAction' => 'View',
        'INPUT.ViewObjectID' => ''
    }), ['EUR']);

    is( $ResultingURL,
        '?ObjectPath=Products/%2208/15%22&Currency=EUR',
        '_TestWithoutViewAction'
    );
}

sub _TestWithPager {
    my $Handler = DE_EPAGES::Presentation::API::TLE::CurrencyHandler->new;
    my $ResultingURL = $Handler->ChangeCurrencyURL(MockedTLEProcessor->new({
        'INPUT.ViewAction' => 'ABC',
        'INPUT.ViewObjectID' => '',
        'Pager' => { URLPage => '?ViewAction=ViewPaged&OrderBy=Alias' }
    }), ['EUR']);

    is( $ResultingURL,
        '?ViewAction=ViewPaged&OrderBy=Alias&Currency=EUR',
        '_TestWithPager'
    );
}

sub _TestWithCurrency {
    my $Handler = DE_EPAGES::Presentation::API::TLE::CurrencyHandler->new;
    my $ResultingURL = $Handler->ChangeCurrencyURL(MockedTLEProcessor->new({
        'INPUT.ViewAction' => 'ABC',
        'INPUT.ViewObjectID' => '',
        'INPUT.QUERY_STRING' => 'ObjectID=1234&ViewAction=ABC&Currency=USD'
    }), ['EUR']);

    is( $ResultingURL,
        '?ObjectPath=Products/%2208/15%22&ViewAction=ABC&Currency=EUR',
        '_TestWithCurrency'
    );
}

sub _TestWithChangeAction {
    my $Handler = DE_EPAGES::Presentation::API::TLE::CurrencyHandler->new;
    my $ResultingURL = $Handler->ChangeCurrencyURL(MockedTLEProcessor->new({
        'INPUT.ViewAction' => 'ABC',
        'INPUT.ViewObjectID' => '',
        'INPUT.QUERY_STRING' => 'ObjectID=1234&ViewAction=ABC&ChangeAction=X&ChangeObjectID=123'
    }), ['EUR']);

    is( $ResultingURL,
        '?ObjectPath=Products/%2208/15%22&ViewAction=ABC&Currency=EUR',
        '_TestWithChangeAction'
    );
}

no warnings 'redefine'; ## no critic qw( ProhibitNoWarnings)
# mock LoadObject()
*DE_EPAGES::Presentation::API::TLE::CurrencyHandler::LoadObject = sub($;$) {
    return MockObject->new({ Path => 'Products/"08/15"' });
};
# mock ExistsObject()
*DE_EPAGES::Presentation::API::TLE::CurrencyHandler::ExistsObject = sub($) {
    return 1;
};
use warnings;

RunScript(
    Sub => sub {
        _TestWithViewAction();
        _TestWithoutViewAction();
        _TestWithPager();
        _TestWithCurrency();
        _TestWithChangeAction();
    },
    Debug => 1
);

package MockedTLEProcessor; ## no critic (ProhibitMultiplePackages)

sub new {
  my $class = shift;
  my ($hVars) = @_;
  return bless $hVars, $class;
}

sub tle {
  my $self = shift;
  my ($TLEVar) = @_;
  return $self->{$TLEVar};
}

package MockObject; ## no critic (ProhibitMultiplePackages)

sub new {
  my $class = shift;
  my ($hAttributes) = @_;
  return bless $hAttributes, $class;
}

sub get {
    my $self = shift;
    my ($Name) = @_;
    return $self->{$Name};
}

1;
