use Test::More;

use strict;

use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::RealexPayment::UI::JSON;
use DE_EPAGES::RealexPayment::t::MockupResponses;

use Digest::SHA1;

RunScript(
    Sub => \&Main
);

sub Main {
    # Test PlaceDelayedSettlement=========================================================

    my $TimeStamp       = '20130517130500';
    my $MerchantID      = 'epages';
    my $OrderID         = '5195FBCBD5809A9B4A96';
    my $PasRef          = '13687839411335182';
    my $AuthCode        = '12345';
    my $comment1        = 'First comment';
    my $comment2        = 'Second comment';
    my $SharedSecret    = 'secret123';
    my $AccountName     = 'internet';

    my $tmp = $TimeStamp . '.' . $MerchantID . '.' . $OrderID . '...';

    my $sha1 = Digest::SHA1->new();
    $sha1->add($tmp);
    my $hash = $sha1->hexdigest();
    my $secretString = "$hash.$SharedSecret";
    $sha1->add($secretString);
    my $sha1Hash = $sha1->hexdigest();

    my $hXMLInfo = {
        TimeStamp => $TimeStamp,
        MerchantID => $MerchantID,
        AccountName => $AccountName,
        OrderID => $OrderID,
        PasRef => $PasRef,
        AuthCode => $AuthCode,
        comment1 => $comment1,
        comment2 => $comment2,
        sha1Hash => $sha1Hash
    };

    # For implementing SKIP_EXTERNAL_TESTS keep in mind that Realex cartridge just works with demo shop of:
    # cs-vm-lin10, cs-vm-lin12 and qa-vm-lin64-1
    my ( $output, $xml ) = DE_EPAGES::RealexPayment::UI::JSON::_createXMLOutput($hXMLInfo); ## no critic qw(ProhibitUnusedVariablesEnhanced ProtectPrivateSubs)

    local *DE_EPAGES::RealexPayment::UI::JSON::_createXMLOutput = ## no critic qw(ProtectPrivateVars)
        *DE_EPAGES::RealexPayment::t::MockupResponses::localCreateXMLOutputMockup;

    my $XMLanswer = DE_EPAGES::RealexPayment::UI::JSON::_createXMLOutput(); ## no critic qw(ProtectPrivateSubs)

    cmp_ok( $output, '==', $XMLanswer, 'XML generated OK!');

    local *DE_EPAGES::RealexPayment::UI::JSON::_doRemoteRealexCall = ## no critic qw(ProtectPrivateVars)
        *DE_EPAGES::RealexPayment::t::MockupResponses::localDoRemoteRealexCallMockup;

    my @ExpectedResponses = qw ( SUCCESS ERROR FAIL );

    foreach my $ExpectedResponse (@ExpectedResponses){
        my $RealexResponse = DE_EPAGES::RealexPayment::UI::JSON::_doRemoteRealexCall($ExpectedResponse); ## no critic qw(ProtectPrivateSubs)

        if ( $ExpectedResponse eq 'SUCCESS' ){
            cmp_ok($RealexResponse, '==', '00', 'Success response works!');
        }
        elsif ( $ExpectedResponse eq 'ERROR' ) {
            cmp_ok($RealexResponse, '>=', '500', 'Error response works!');
        }
        elsif ( $ExpectedResponse eq 'FAIL' ) {
            cmp_ok($RealexResponse, '!=', '200', 'Fail response works!');
        }
    }
    done_testing;
}